function [mask,topLeftCorner, spotCentroids] = arrayGridMask(im,pxSize,topLeftCorner,debugFlag)
% Creates a binary mask of microarray spots for a microarray of size
% 7002x10350 mum with 96 rows and 82*2 columns. The spot radius is only 90%
% of the real radius to deal with possible missalignment of the mask with
% images
%
% Input:
%   im - preprocessed fluo image.
%   pxSize - pixel size in mum.
%   topLeftCorner - [x,y] coordinates of the top left spot centroid.
%                   Optional, if not provided or empty, the centroid is
%                   set using gui.  Default [].
%   debugFlag - optional, if true plot image with spot outlines. Default
%               false.

NROWS = 96;
NCOLS = 82;

colSpacing = 0.127*1000/pxSize;
rowSpacing = 0.073323*1000/pxSize;
radius = round(0.9*10*pxSize);

if nargin<3 || isempty(topLeftCorner)
    figure, imshow(imadjust(im(1:round(10*rowSpacing),1:round(10*colSpacing))));
    [x,y] = ginput(1);
    topLeftCorner = [x y];
end
if nargin<4
    debugFlag = false;
end

spotCentroidsX = topLeftCorner(1):colSpacing:colSpacing*NCOLS;
spotCentroidsY = topLeftCorner(2):rowSpacing:rowSpacing*NROWS;
spotCentroids = zeros(NROWS,NCOLS*2,2);

mask = false(size(im));
[XX, YY] = meshgrid(spotCentroidsX, spotCentroidsY);
spotCentroids(:,1:2:end,1) = round(XX);
spotCentroids(:,1:2:end,2) = round(YY);
rcenters = round([XX(:) YY(:)]);
[XX, YY] = meshgrid(spotCentroidsX+colSpacing/2, spotCentroidsY+rowSpacing/2);
rcenters = [rcenters; round([XX(:) YY(:)])];
mask(sub2ind(size(mask),rcenters(:,2),rcenters(:,1))) = 1;
spotCentroids(:,2:2:end,1) = round(XX);
spotCentroids(:,2:2:end,2) = round(YY);
se = strel('disk',radius,8);
mask = imdilate(mask,se);
if debugFlag
    figure, imshow(imadjust(im)), hold on
    B = bwboundaries(mask,'noholes');
    visboundaries(B,'EnhanceVisibility',0,'LineWidth',1);
end