function [rates] = modelRatesVecExtended(kaMax,kOffMicroDeep,pvals,pstarvals)
%This function return rates=[kd,ka] for a multistep sequential binding
%process parameterised by (kaMax,kOffMicroDeep,pvals), where pvals is a
%vector with probabilities, pvals(i)= probability to transiation from state
%i+1 to i, where state 1 is the completeley specifically bound state (with microscopic offrate kOffMicroDeep)
%Extended:
%pstarvals= The probability NOT to  dissocatie at this testing state = pstar,
%so that 1-pstar is the probaiblity to disocate macroscopically directly from this
%state. pstarvals(i) is this probability at testing state i+1
nsteps=numel(pvals);

dim=nsteps+2;
timesOn=zeros(dim,1);
timesOn(end)=1/kaMax;

timesOff=zeros(dim,1);
timesOff(1)=1/kOffMicroDeep;

for i=1:nsteps-1
    if pvals(i)>pstarvals(i)
        pstarvals(i)=pvals(i);
    end
    
end


transMatr=genStepTransMatrExtended(pvals,pstarvals);

[onMatr,offMatr] = getTimeMatrExtended(transMatr);

%if (rcond(onMatr)<eps||rcond(offMatr)<eps)
%    rates=[inf,inf];
%    
%else 

lastwarn('');
solutionOn=onMatr\timesOn;
solutionOff=offMatr\timesOff;
if ~isempty(lastwarn())
    ka=inf;
    kd=inf;
else
    ka=1./solutionOn(end);
    kd=1./solutionOff(1);

end        
rates=[kd,ka];
end

