function [trajectories,reactions,dnaCoords,X,T] = runOnOffSimulation(kdVec,kaVec,protconc,numSitesVec,xDim,yDim,names,T_dt,tmax,equillibrated)
%This function runs an binding/unbinding experimen, returns trajectores and
%reactions suitible for export to be ued as import for SMeagol to do
%simulated microscopy

%time inbetween two time-points at which the simulation state is saved
%T_dt = 1/(60*60);
% Column vector of time points at which the simulation state is saved
T = (0:T_dt:tmax)';
tspan= [T(1) T(end)];

numSiteTypes=numel(numSitesVec);
totNumSites=sum(numSitesVec);
% Matrix defining the change in copy numer for each reaction. See function
% for a more in-depth description


S =sparse(2*totNumSites,totNumSites);

for i=1:2*totNumSites
   if mod(i,2)==1 
   S(i,ceil(i/2))=-1; %Dissocation
   
   else
   S(i,i/2)=1; %Assocation
   
   end
end

kdVecAll=zeros(1,totNumSites);
kaVecAll=zeros(1,totNumSites);
namesVecAll=cell(1,totNumSites);
k=1;

for i=1:numSiteTypes
    kdVecAll(k:k+numSitesVec(i)-1)=kdVec(i);
    kaVecAll(k:k+numSitesVec(i)-1)=kaVec(i);
    for j=0:numSitesVec(i)-1
        namesVecAll{k+j}=names{i};
    end
    k=k+numSitesVec(i);
end


% Matrix for storing the sampled results.
X = zeros(size(T,1), totNumSites);

% current state defined by the copy number of each components is save in x.
x0=zeros(1,totNumSites);
if equillibrated==0 % Association experiment
 
elseif equillibrated==1 %equillibrate at start
   for i=1:totNumSites
        occupancy=(1/kdVecAll(i))/((1/kdVecAll(i))+(1/kaVecAll(i)));
        x0(i)=rand()<occupancy;
   end
end
x=x0;
currXInds=zeros(1,totNumSites);



%Initialise currXInds at time zero, have to increase inds if molecules
%are present at time 0
    
    
currIncreaseInd=sum(x)+1;
currXInds(x==1)=1:sum(x);
    


%Ti defines an ndex of where in T one should write a sample
Ti=1;




% simulation time is kept in t.
t = 0;


Ti=1;

%Intitiate binding sites positions
positions=zeros(totNumSites,3);

for i=1:totNumSites
    
    positions(i,1)=xDim*rand();
    positions(i,2)=yDim*rand();
    positions(i,3)=0;
end

dnaCoords=positions(1:numSitesVec(1),:);

trajectories=cell(1,numel(T));
%reactions=table('Size',[100000 6],'VariableTypes',{'double','double','string','double','double','double'});%time molInd speciesname x y z
reactions=cell(1000000,6);
numReactions=0;

%also Index molcules in order of appearance, not only by there DNA site



% The simulation will be stopped after the system state has been saved at
% the last time point in T.
is_running=true;
while is_running
    % r is a vector containing the rate for each reaction
    r =zeros(1,2*totNumSites);
    k=1;
    for i=1:2:2*totNumSites
        if x(k)==1 %Dissocation possible
            r(i)=kdVecAll(k);
        elseif x(k)==0
            r(i+1)=kaVecAll(k)*protconc;% assocation possible
        end
        k=k+1;
    end
    
    
    %sample the time until any evet will happen
    trxn = exprnd(1/sum(r));
    
    t = t + trxn;
    
    %Save state at regular intervals
    while t>= T(Ti)
        X(Ti,:) =x;
        
        numAlive=sum(x);
        siteInds=find(x==1);
        moleculeInds=currXInds(siteInds);
        [sortMolInds,sortedInds]=sort(moleculeInds);
        trajectories{Ti}=cell(numAlive,5);
        
        for i=1:numAlive
            trajectories{Ti}{i,1}=sortMolInds(i); %Molecule Index
            trajectories{Ti}{i,2}=namesVecAll{siteInds(sortedInds(i))}; %Molecule Index
            trajectories{Ti}{i,3}=positions(siteInds(sortedInds(i)),1);%x position
            trajectories{Ti}{i,4}=positions(siteInds(sortedInds(i)),2);%y position
            trajectories{Ti}{i,5}=positions(siteInds(sortedInds(i)),3);%z positio
            
            
        end
        
        Ti=Ti+1;
        
      
        if Ti == length(T) +1
              % Stop the simulation
            is_running = false;
   
    %        %Make sure to break the current while loop (otherwise T(Ti)
    %        % will point outside the vector.
            break;
        end
      
        
    end
    
    if t>tmax
        is_running=false;
        reactions=reactions(1:numReactions,:);
        break;
    end
    
    % sample which reaction that occured.
    sampled_rxn = find(rand() <= cumsum(r)./sum(r),1);
  
    
    
    % update the state according the sampled reaction the change is defined
    % by the matrix S.
    x = x + S(sampled_rxn,:);
    
   
   rx_siteInds=find(S(sampled_rxn,:)~=0);
   rx_molInds=currXInds(rx_siteInds);
   
   
   rx_type=S(sampled_rxn,rx_siteInds);
   
   for i=1:numel(rx_type)
       numReactions=numReactions+1;
       
       if rx_type(i)==1
           
           
           reactions{numReactions,1}=t;
           reactions{numReactions,2}=currIncreaseInd;
           currXInds(rx_siteInds(i))=currIncreaseInd;
           currIncreaseInd=currIncreaseInd+1;%A new molecule was created, increase the index count
           reactions{numReactions,3}=namesVecAll{rx_siteInds(i)};
           reactions{numReactions,4}=positions(rx_siteInds(i),1);
           reactions{numReactions,5}=positions(rx_siteInds(i),2);
           reactions{numReactions,6}=positions(rx_siteInds(i),3);
           
           
       elseif rx_type(i)==-1
            reactions{numReactions,1}=t;
           reactions{numReactions,2}=rx_molInds(i);
           reactions{numReactions,3}=-1;
           reactions{numReactions,4}=positions(rx_siteInds(i),1);
           reactions{numReactions,5}=positions(rx_siteInds(i),2);
           reactions{numReactions,6}=positions(rx_siteInds(i),3);
       end
   end
    
   
    
end




end

