function [updatedTrackedCells] = cleanBaxterOutput(trackedCells,pathSegImages)
% Removes the cells that have too short lifetime and then adds a unique id
% to each cell as well as the labels of the cell in each frame it is alive
%
% Inputs :
%       trackedCells  : Baxter output structure.
%       pathSegImages : Path to the segmented images.
%
% Output : 
%       updatedTrackedCells : Updated Baxter output structure.


segImagesList = dir(strcat(pathSegImages,filesep,'*.tif*'));
id = 1;

% Remove objects that Baxter do not consider to be cells and objects which
% lifetime is too small to be a cell. Then assign them a unique id.
for trackedCellIdx=1:length(trackedCells)
    if trackedCells(trackedCellIdx).lifeTime > 5 && trackedCells(trackedCellIdx).isCell
        updatedTrackedCells(id) = trackedCells(trackedCellIdx);
        updatedTrackedCells(id).id = id;
        id = id+1;
    end
end

% Removes the parents and children fields if their id is set to 0 (i.e.
% they did not pass the previous check)
for trackedCellIdx=1:length(updatedTrackedCells)
    if ~isempty(updatedTrackedCells(trackedCellIdx).parent)
        if updatedTrackedCells(trackedCellIdx).parent.id == 0
            updatedTrackedCells(trackedCellIdx).parent = [];
        end
    end
    
    for childrenIdx = length(updatedTrackedCells(trackedCellIdx).children):-1:1
        if updatedTrackedCells(trackedCellIdx).children(childrenIdx).id == 0
            updatedTrackedCells(trackedCellIdx).children(childrenIdx) = [];
        end
    end
    
    mesh = [];
     % Get the label of the cell in each frame it exists
    for jj=updatedTrackedCells(trackedCellIdx).firstFrame:updatedTrackedCells(trackedCellIdx).lastFrame
        currIm = imread(strcat(pathSegImages,filesep,segImagesList(jj).name));
        currMesh = currIm(round(updatedTrackedCells(trackedCellIdx).cy(jj-updatedTrackedCells(trackedCellIdx).firstFrame+1)),round(updatedTrackedCells(trackedCellIdx).cx(jj-updatedTrackedCells(trackedCellIdx).firstFrame+1)));
        
        mesh = [mesh currMesh];
    end
    
    updatedTrackedCells(trackedCellIdx).mesh = mesh;
end
