function [cellWidth] = computeCellWidth(inputImage, currLabel, backBoneStruct)
% Computes the width of a cell by computing several normal vectors to the
% backbone and measuring their intersection with the cell, then taking the
% maximum of all computed distances.

    % Compute the maximum and minimum coordinates of the cell and create
    % the label image
    if ~backBoneStruct.isRotated
        minCoord = min(backBoneStruct.Coordinates(:,2))-0.5;% remove 0.5 to be at the edge of the origin pixel
        maxCoord = max(backBoneStruct.Coordinates(:,2))+0.5;% add 0.5 to be at the edge of the last pixel
        
        minCoordY = min(backBoneStruct.Coordinates(:,1))-0.5;% remove 0.5 to be at the edge of the origin pixel
        maxCoordY = max(backBoneStruct.Coordinates(:,1))+0.5;% add 0.5 to be at the edge of the last pixel
        
        currentIm = inputImage;
        labelIm = currentIm == currLabel;
    else
        minCoord = min(backBoneStruct.Coordinates(:,1))-0.5;% remove 0.5 to be at the edge of the origin pixel
        maxCoord = max(backBoneStruct.Coordinates(:,1))+0.5;% add 0.5 to be at the edge of the last pixel
        
        [~, minIdx] = min(backBoneStruct.Coordinates(:,2));
        [~, maxIdx] = max(backBoneStruct.Coordinates(:,2));
        
        minRotated = apply90RotCcwToPoint(size(inputImage), [backBoneStruct.Coordinates(minIdx,1) backBoneStruct.Coordinates(minIdx,2)]);
        maxRotated = apply90RotCcwToPoint(size(inputImage), [backBoneStruct.Coordinates(maxIdx,1) backBoneStruct.Coordinates(maxIdx,2)]);
        
        minCoordY = min(minRotated(1),maxRotated(1))-0.5;% remove 0.5 to be at the edge of the origin pixel
        maxCoordY = max(minRotated(1),maxRotated(1))+0.5;% add 0.5 to be at the edge of the last pixel
        
        currentIm = rot90(inputImage);
        labelIm = currentIm == currLabel;
    end
    
    step = ceil((maxCoord - minCoord)/15);
    widthIdx = 1;
    
    for backBoneIdx = (minCoord):step:(maxCoord)
        % Compute the local linear approximation of the backbone fit
        slope = backBoneStruct.Fit.p1*2*backBoneIdx+backBoneStruct.Fit.p2;
        firstPoint(1) = backBoneIdx;
        firstPoint(2) = backBoneStruct.Fit(backBoneIdx);
        
        tangentVector(1) = 1;
        tangentVector(2) = slope;
        
        % get the orthogonal vector
        orthogonalVector = [slope -1];
        
        % Compute the interval needed for sampling the orthogonal fit
        tempInterval = [-200:1:max(size(currentIm))];
        tempCoord = (orthogonalVector(2)/orthogonalVector(1)*tempInterval+firstPoint(2))';
        minVal = min(tempInterval(logical([tempCoord>=minCoordY-20].*[tempCoord<=maxCoordY+20])));
        maxVal = max(tempInterval(logical([tempCoord>=minCoordY-20].*[tempCoord<=maxCoordY+20])));
        roundedMinVal = round(minVal);
        
        if roundedMinVal == round(maxVal)
            tempInterval = [roundedMinVal-5:0.00002:roundedMinVal+5];
            tempCoord = (orthogonalVector(2)/orthogonalVector(1)*tempInterval+firstPoint(2))';
            minVal = min(tempInterval(logical([tempCoord>=minCoordY-30].*[tempCoord<=maxCoordY+30])));
            maxVal = max(tempInterval(logical([tempCoord>=minCoordY-30].*[tempCoord<=maxCoordY+30])));
            
            interval = minVal:0.00002:maxVal;
        else
            interval = minVal:0.005:maxVal;
        end
        
        % Compute the points on the line that are also part of the cell
        if ~backBoneStruct.isRotated
            allCoord = (orthogonalVector(2)/orthogonalVector(1)*interval+firstPoint(2))';
            roundedCoord = unique(round([(interval+backBoneIdx)' allCoord]),'rows');
            roundedCoord(roundedCoord(:,2)<1,:) = [];
            roundedCoord(roundedCoord(:,2)>size(currentIm,1),:) = [];
            roundedCoord(roundedCoord(:,1)>size(currentIm,2),:) = [];
            roundedCoord(roundedCoord(:,1)<1,:) = [];
            linIdx = sub2ind(size(currentIm),roundedCoord(:,2),roundedCoord(:,1));
            fitIm = zeros(size(currentIm));
            fitIm(linIdx) = 1;
            ribIm = labelIm.*fitIm;
        else
            allCoord = (orthogonalVector(2)/orthogonalVector(1)*interval+firstPoint(2))';
            roundedCoord = unique(round([(interval+backBoneIdx)' allCoord]),'rows');

            roundedCoord(roundedCoord(:,2)<1,:) = [];
            roundedCoord(roundedCoord(:,2)>size(currentIm,1),:) = [];
            roundedCoord(roundedCoord(:,1)>size(currentIm,2),:) = [];
            roundedCoord(roundedCoord(:,1)<1,:) = [];
            linIdx = sub2ind(size(currentIm),roundedCoord(:,2),roundedCoord(:,1));
            fitIm = zeros(size(currentIm));
            fitIm(linIdx) = 1;
            ribIm = labelIm.*fitIm;
        end
        ribTempCoord = find(ribIm);
        [ribCoord(:,1), ribCoord(:,2)] = ind2sub(size(currentIm),ribTempCoord);
        
        if isempty(ribCoord)
            cellWidth = -1;
            return
        end
        
        % Sort the coordinates
        [~, maxIdxX] = max(ribCoord(:,1));
        [~, minIdxX] = min(ribCoord(:,1));
        
        [~, maxIdxY] = max(ribCoord(:,2));
        [~, minIdxY] = min(ribCoord(:,2));
        
        % Computing the cell width: euclidean distance between the end points
        widthX = sqrt((ribCoord(maxIdxX(1),1)+0.5-(ribCoord(minIdxX(1),1)-0.5))^2+(ribCoord(maxIdxX(1),2)+0.5-(ribCoord(minIdxX(1),2)-0.5))^2);
        widthY = sqrt((ribCoord(maxIdxY(1),1)+0.5-(ribCoord(minIdxY(1),1)-0.5))^2+(ribCoord(maxIdxY(1),2)+0.5-(ribCoord(minIdxY(1),2)-0.5))^2);
        
        tempCellWidth(widthIdx) = max(widthX,widthY);
 
        widthIdx = widthIdx+1;
        clear ribCoord;
    end
    cellWidth = max(tempCellWidth);
end