function [countLongAxis, countShortAxis] = displayForkPlot(analysisFolder, mirrorFlag)
%% Parameters
scale = 40/1178;% pixel size, in um
minCellLength = 30;

%% Initialisation
posList = dir(strcat(analysisFolder,filesep,'Pos*'));
indexOutLong = 1;

%% Data extraction
for posIdx = 1:length(posList)
    currPos = posList(posIdx).name;
    
    listMat = dir(strcat(analysisFolder,filesep,currPos,filesep,'*',currPos,'.mat'));
    cellFile = load(strcat(analysisFolder,filesep,currPos,filesep,listMat(1).name));
    fieldNames = fieldnames(cellFile);
    cellsArray = cellFile.(fieldNames{1});
    
    disp(['Processing cells in ' currPos])
    for ii = length(cellsArray):-1:1
        if ~isempty(cellsArray(ii).cellLength)
            currCell = cellsArray(ii);
            if currCell.cellLength > minCellLength && currCell.cellWidth > 0
                outArrayLong{indexOutLong,1} = currCell.cellLength*scale; % scale to be in um
                outArrayShort{indexOutLong,1} = currCell.cellWidth*scale; % scale to be in um
                
                for jj = 1:length(currCell.particles)
                    outArrayLong{indexOutLong,jj+1} = currCell.particles(jj).trajInternalCoord(1);
                    outArrayShort{indexOutLong,jj+1} = currCell.particles(jj).trajInternalCoord(2);
                end
                indexOutLong = indexOutLong+1;
            end
        end
    end
end

%% Long axis
minCellLength = min([outArrayLong{:,1}]);
maxCellLength = max([outArrayLong{:,1}]);
yBin = 31;
xBin = 0.04;
yCenter = maxCellLength*0.5*yBin;

binning = minCellLength:xBin:maxCellLength-3;

xTicks = 0.2/xBin:10:length(binning);
format bank
xTickLabels = binning(xTicks)-0.01;%minCellLength:maxCellLength;
yTickLabels = -5:5;
yTicks = yBin*yTickLabels+yCenter;

disp('Displaying stuff')
% Creating heatmap
[countsLong, idxLong] = histc([outArrayLong{:,1}],binning);

heatMapLong = zeros(ceil(maxCellLength*yBin),length(countsLong));
heatMapLong2 = zeros(ceil(maxCellLength*yBin),length(countsLong));

for ii = 1:length(countsLong)
    currArray = [outArrayLong(idxLong==ii,:)];
    
    for jj = 1:length([currArray{:,1}])
        for kk = 2:length([currArray{jj,:}])
            xIndx = ceil(((currArray{jj,kk}-0.5)*currArray{jj,1}*yBin)+yCenter)-1;
            heatMapLong(xIndx,ii) = heatMapLong(xIndx,ii)+1; 

            if mirrorFlag
                mirrorIndx = ceil((-(currArray{jj,kk}-0.5)*currArray{jj,1}*yBin)+yCenter)-1;
                heatMapLong(mirrorIndx,ii) = heatMapLong(mirrorIndx,ii)+1; 
            end
        end
    end
    clear currArray;
%     heatMapLong2(:,ii) = heatMapLong(:,ii)./countsLong(ii);
end

maxHeatMap = max(max(heatMapLong));


% Adding the cell length lines
for ii = 2:length(countsLong)
    minIndx = ceil(((1-0.5)*binning(ii)*yBin)+yCenter)-1;
    maxIndx = ceil(((0-0.5)*binning(ii)*yBin)+yCenter)-1;
    
    heatMapLong2(:,ii) = heatMapLong(:,ii)*(minIndx-maxIndx);

    heatMapLong(minIndx,ii) = maxHeatMap+1;
    heatMapLong(maxIndx,ii) = maxHeatMap+1;
end

maxHeatMap2 = max(max(heatMapLong2));

for ii = 2:length(countsLong)    
    minIndx = ceil(((1-0.5)*binning(ii)*yBin)+yCenter)-1;
    maxIndx = ceil(((0-0.5)*binning(ii)*yBin)+yCenter)-1;

    heatMapLong2(minIndx,ii) = maxHeatMap2+1;
    heatMapLong2(maxIndx,ii) = maxHeatMap2+1;
end

%% Short axis
minCellLengthShort = min([outArrayShort{:,1}]);
maxCellLengthShort = max([outArrayShort{:,1}]);
yBinShort = 35;
xBinShort = 0.02;
yCenterShort = maxCellLengthShort*0.5*yBinShort;

binningShort = minCellLengthShort+0.3:xBinShort:maxCellLengthShort-0.3;

xTicksShort = 0.02/xBinShort:2:length(binningShort);
format bank
xTickLabelsShort = binningShort(xTicksShort);%minCellLength:maxCellLength;
yTickLabelsShort = -3:0.1:3;
yTicksShort = yBinShort*yTickLabelsShort+yCenterShort;

disp('Displaying stuff')
% Creating heatmap
[countsShort, idxShort] = histc([outArrayShort{:,1}],binningShort);

heatMapShort = zeros(ceil(maxCellLengthShort*yBinShort),length(countsShort));
heatMapShort2 = zeros(ceil(maxCellLengthShort*yBinShort),length(countsShort));

for ii = 1:length(countsShort)
    currArray = [outArrayShort(idxShort==ii,:)];
    
    for jj = 1:length([currArray{:,1}])
        for kk = 2:length([currArray{jj,:}])
            xIndx = max(1,ceil(((currArray{jj,kk})*currArray{jj,1}*yBinShort/2)+yCenterShort)-1);
            xIndx = min(ceil(maxCellLengthShort*yBinShort), xIndx);
            heatMapShort(xIndx,ii) = heatMapShort(xIndx,ii)+1; 
            
            if mirrorFlag
                mirrorIndx = max(1,ceil(-((currArray{jj,kk})*currArray{jj,1}*yBinShort/2)+yCenterShort)-1);
                mirrorIndx = min(ceil(maxCellLengthShort*yBinShort), mirrorIndx);
                heatMapShort(mirrorIndx,ii) = heatMapShort(mirrorIndx,ii)+1;
            end
        end
    end
    clear currArray;
end

maxHeatMap = max(max(heatMapShort));

% Adding the cell length lines
for ii = 2:length(countsShort)
    minIndx = ceil((binningShort(ii)*yBinShort/2)+yCenterShort)-1;
    maxIndx = ceil((-binningShort(ii)*yBinShort/2)+yCenterShort)-1;

    heatMapShort2(:,ii) = heatMapShort(:,ii)*(minIndx-maxIndx);
    
    heatMapShort(minIndx,ii) = maxHeatMap+1;
    heatMapShort(maxIndx,ii) = maxHeatMap+1;
end

maxHeatMapShort2 = max(max(heatMapShort2));

for ii = 2:length(countsShort)
    minIndx = ceil((binningShort(ii)*yBinShort/2)+yCenterShort)-1;
    maxIndx = ceil((-binningShort(ii)*yBinShort/2)+yCenterShort)-1;
    
    heatMapShort2(minIndx,ii) = maxHeatMapShort2+1;
    heatMapShort2(maxIndx,ii) = maxHeatMapShort2+1;
    
end

%% Updating output variables
countLongAxis = uint32(sum(countsLong));
countShortAxis = uint32(sum(countsShort));

%% Plotting
% Long axis
figure, imagesc(heatMapLong2)

colormap(hot)
set(gca,'XTickLabel',sprintf('%1.2f|',xTickLabels))
set(gca,'XTick',xTicks)
set(gca,'YTickLabel',yTickLabels)
set(gca,'YTick',yTicks)
a = gca;
b = copyobj(a,gcf);
set(b,'XColor',[1 1 1], 'YColor', [1 1 1], 'XTickLabel', [], 'YTickLabel', []);
xlabel('Binned length of the cell, in um')
ylabel('Location along the long axis, in um')

% Short axis
figure, imagesc(heatMapShort2)

colormap(hot)
set(gca,'XTickLabel',sprintf('%1.2f|',xTickLabelsShort))
set(gca,'XTick',xTicksShort)
set(gca,'YTickLabel',yTickLabelsShort)
set(gca,'YTick',yTicksShort)
a = gca;
b = copyobj(a,gcf);
set(b,'XColor',[1 1 1], 'YColor', [1 1 1], 'XTickLabel', [], 'YTickLabel', []);
xlabel('Binned width of the cell, in um')
ylabel('Location along the short axis, in um')


% %% Display number of dots as a function of cell length
% 
% for cellIdx = 1:length([outArrayLong{:,1}])
%     dotPerLength(cellIdx,:) = [outArrayLong{cellIdx,1} length([outArrayLong{cellIdx,2:end}])];
% end
% 
% 
% for ii = 1:length(countsLong)
%     temp = [dotPerLength(idxLong==ii,2)];
%     currArray(ii) = mean(temp(temp>0));
% end
% 
% figure, plot(currArray,'+')
