function [coeffDiff,coeffDiffComponents,sigmas,varDComponents] = getDiffCVEanderror(trajs,timeStep,pixelSize,Dimension,exptime)
 %Calculculates diff coeff from trajectories according to Vestegaard et al (CVE estimator)
    %inputs: timeStep is the total tiem step per frame, exptime is the
    %exposure time that the camera shutter is open (for calc of R)
    %-output: coeffDiff = Overall Diff coeff (mean of all dimensions).
    % DiffComponents = The components for every dimension contributing to
    % the the total diff. coeff.
    % sigmas = standard deviation of the localization error for everu
    % trajectory and dimension
    % varDComponents = variance of estimated diffusion coefficients for
    % every trajectory in the different dimensions.
    
numtrajs=length(trajs);
coeffDiffComponents=zeros(Dimension,numtrajs);
sigmas=zeros(Dimension,numtrajs);
varDComponents=zeros(Dimension,numtrajs);
coeffDiff=zeros(1,numtrajs);
R = exptime/(timeStep*6);
    for i=1:numtrajs
        for j=1:Dimension
            thediffs=diff(trajs{i}(:,j))*pixelSize*10^-3;
            N=length(thediffs);
            meandisp=mean((thediffs).^2);
            meancorr=mean(thediffs(1:end-1).*thediffs(2:end));
            coeffDiffComponents(j,i)=meandisp/(2*timeStep*10^-3)+...
               meancorr/(timeStep*10^-3);
            sigmas(j,i)= R*meandisp+(2*R-1)*meancorr;
            epsi=sigmas(j,i)/(coeffDiffComponents(j,i)*timeStep*10^-3)-2*R;
            varDComponents(j,i)=coeffDiffComponents(j,i)^2*((6+4*epsi+2*(epsi^2))/N+(4*((1+epsi)^2))/(N^2)); 
        end
        coeffDiff(i)=mean(coeffDiffComponents(:,i));
    end
    sigmas=sqrt(sigmas);
end