function [forkMap, figureHandle, normedForkMap] = getForkPlot(cellArrayFile, sizeTrapInPixels)

scale = 40/sizeTrapInPixels;% pixel size, in um

longCellFile = load(cellArrayFile);
fieldNameLong = fieldnames(longCellFile);
longCellStruct = longCellFile.(fieldNameLong{1});

indexOut = 1;

disp('Processing fast cells')
for ii = length(longCellStruct):-1:1
    if ~isempty(longCellStruct(ii).cellLength)
        currCell = longCellStruct(ii);
        if currCell.cellLength > 30
            outArrayLong{indexOut,1} = currCell.cellLength*scale; % converts length from pixels to um
            for jj = 1:length(currCell.particles)
                if abs(currCell.particles(jj).trajInternalCoord(1)) <= 1 && abs(currCell.particles(jj).trajInternalCoord(2)) <= 1
                    outArrayLong{indexOut,jj+1} = currCell.particles(jj).trajInternalCoord(1);
                end
            end
            indexOut = indexOut+1;
        end
    end
end

clear longCellStruct

minCellLength = min([outArrayLong{:,1}]);
maxCellLength = max([outArrayLong{:,1}]);
yBin = 30;
xBin = 0.04;
yCenter = maxCellLength*0.5*yBin;

binning = minCellLength:xBin:maxCellLength;

xTicks = 0.2/xBin:10:length(binning);
format bank
xTickLabels = binning(xTicks);%minCellLength:maxCellLength;
yTickLabels = -5:5;
yTicks = yBin*yTickLabels+yCenter;

disp('Displaying stuff')
% Creating heatmap
[countsLong, idxLong] = histc([outArrayLong{:,1}],binning);

forkMap = zeros(ceil(maxCellLength*yBin),length(countsLong));
normedForkMap = zeros(ceil(maxCellLength*yBin),length(countsLong));

for ii = 1:length(countsLong)
    currArray = outArrayLong(idxLong==ii,:);
    
    for jj = 1:length([currArray{:,1}])
        for kk = 2:length([currArray{jj,:}])
            xIndx = ceil(((currArray{jj,kk}-0.5)*currArray{jj,1}*yBin)+yCenter)-1;
            forkMap(xIndx,ii) = forkMap(xIndx,ii)+1; 
        end
    end
    clear currArray;
    normedForkMap(:,ii) = forkMap(:,ii)./countsLong(ii); % Same heatmap but normalized by number of cells in each bin
end

maxHeatMap = max(max(forkMap));

for ii = 2:length(countsLong)
    minIndx = ceil(((1-0.5)*binning(ii)*yBin)+yCenter)+1;
    maxIndx = ceil(((0-0.5)*binning(ii)*yBin)+yCenter)+1;
    forkMap(minIndx,ii) = maxHeatMap+1;
    forkMap(maxIndx,ii) = maxHeatMap+1;
    
    normedForkMap(minIndx,ii) = maxHeatMap+1;
    normedForkMap(maxIndx,ii) = maxHeatMap+1;
    
end

figureHandle = figure;
imagesc(forkMap), title('Slow growing cells')

colormap(hot)
set(gca,'XTickLabel',sprintf('%1.2f|',xTickLabels))
set(gca,'XTick',xTicks)
set(gca,'YTickLabel',yTickLabels)
set(gca,'YTick',yTicks)
a = gca;
b = copyobj(a,gcf);
set(b,'XColor',[1 1 1], 'YColor', [1 1 1], 'XTickLabel', [], 'YTickLabel', []);
xlabel('Binned length of the cell, in um')
ylabel('Location along the main axis, in um')
format longG