function [projectedPoint, distToProj] = getProjectedPoint(backBoneStruct,pointCoord)
% Compute the projected point from a point in the plane onto a polynomial
% curve.
%
% Inputs
%   backBoneStruct : Structure of the backbone of the cell. Obtained with
%                    computeSingleCellFit.
%   pointCoord     : Coordinates of the point to be projected onto the
%                    curve.
%
% Output
%   projectedPoint : Coordinates of the projected point pointCoord onto the
%                    curve.
%   distToProj     : Distance between pointCoord and its projection.

    if ~backBoneStruct.isRotated
        minCoord = min(backBoneStruct.Coordinates(:,2));
        maxCoord = max(backBoneStruct.Coordinates(:,2));
    else
        minCoord = min(backBoneStruct.Coordinates(:,1));
        maxCoord = max(backBoneStruct.Coordinates(:,1));
    end
    
    coordRange = [minCoord:0.1:maxCoord]';
    fitValues = backBoneStruct.Fit(coordRange);
    
    % For our data (big sampling of the backbone and single point to
    % project) it seems that not using a delaunay triangulation in the
    % dsearchn function is much faster and uses much less memory
    [closestPointIdx, distToProj] = dsearchn([coordRange fitValues],pointCoord);
    
    projectedPoint = [coordRange(closestPointIdx) fitValues(closestPointIdx)];
 end