function rescaleImages(pathImages, outputPath)
% Function to rescale the images to use the full range of values for uint16
% images.
    if nargin < 2
        outputPath = pathImages;
    end

    list = dir(strcat(pathImages,filesep,'*.tif'));
    tiffBool = false;
    
    if isempty(list)
        list = dir(strcat(pathImages,filesep,'*.tiff'));
        tiffBool = true;
        if isempty(list)
            disp('The file format is neither .tif or .tiff, aborting.');
            return;
        end
    end

    % Compute the maximum and the minimum values in the dataset
    maxVal = -1;
    minVal = 65536;
    for ii = 1:length(list)
        im = uint16(imread(strcat(pathImages,filesep,list(ii).name)));
        if max(max(im)) > maxVal
            maxVal = max(max(im));
        end
        if min(min(im)) < minVal
            minVal = min(min(im));
        end
    end

    % Compute the ratio used to rescale the images
    newRatio = 65535/(maxVal-minVal);

    % Rescale the images and save them as uint16
    for ii = 1:length(list)
        im = uint16(imread(strcat(pathImages,filesep,list(ii).name)));
        tempIm = (im-minVal).*newRatio;
        if tiffBool
            imwrite(tempIm,strcat(outputPath,filesep,'rescaled_',list(ii).name(1:end-1)));
        else
            imwrite(tempIm,strcat(outputPath,filesep,'rescaled_',list(ii).name(1:end)));
        end
    end
end