function [] = segmentFluoAndCorrectDots(sourceFolder,outputFolder, pathScript, parameterFile)
%
% Inputs:
%   sourceFolder  : where the images are.
%   outputFolder  : where the results will be saved.
%   pathScript    : address + name of the python script (.py).
%   parameterFile : address + name of the parameter file (.txt).
%

%% Initialisation
    positionList = dir(strcat(sourceFolder,filesep,'Pos*'));
    se = strel('disk',3,8);
    
    if nargin<4
        parameters = parseParameterFile();
    else
        parameters = parseParameterFile(parameterFile);
    end
    
    %% Preprocessing
    disp('Preprocessing.');
    for positionIdx = 1:length(positionList)-1
        disp(positionList(positionIdx).name);
        
        % Creating the different folders needed
        mkdir(outputFolder,positionList(positionIdx).name);
        mkdir(strcat(outputFolder,filesep,positionList(positionIdx).name),'ComplementedImages');
        mkdir(strcat(outputFolder,filesep,positionList(positionIdx).name),'SegmentedImages');
        
        complementImages(strcat(sourceFolder,filesep,positionList(positionIdx).name,filesep,'fluor_514'),strcat(outputFolder,filesep,positionList(positionIdx).name,filesep,'ComplementedImages'));
        
    end

    %% Segmentation
    disp('Segmenting.');
    for positionIdx = 1:length(positionList)-1
        disp(positionList(positionIdx).name)
        sysCommand = ['python2.7 ' pathScript ' ' strcat(outputFolder,filesep,positionList(positionIdx).name,filesep,'ComplementedImages',filesep) ' ' strcat(outputFolder,filesep,positionList(positionIdx).name,filesep,'SegmentedImages',filesep) ' ' int2str(parameters.segFirstFrame) ' ' int2str(parameters.minWidth) ' ' int2str(parameters.maxWidth) ' ' int2str(parameters.minLength) ' ' int2str(parameters.maxLength)];
        [status, result] = system(sysCommand);
        
        if status ~= 0
            disp('There was an error during segmentation, below is the output of the segmentation script: ');
            result
            return
        end
    end
   
    %% Postprocessing
    disp('Post processing.');
    for positionIdx = 1:length(positionList)-1
        disp(positionList(positionIdx).name);
        listSeg = dir(strcat(outputFolder,filesep,positionList(positionIdx).name,filesep,'SegmentedImages/*.tif*'));
        
        
        for jj = 1: length(listSeg)
            currSegIm = imread(strcat(outputFolder,filesep,positionList(positionIdx).name,filesep,'SegmentedImages',filesep,listSeg(2).name));

            outIm = zeros(size(currSegIm));
            for ii =1:max(max(currSegIm))
                currIm = currSegIm==ii;
                dilIm = imclose(currIm,se);
                outIm = max(outIm,dilIm);
            end
            outIm = uint16(bwlabel(uint16(outIm),4));
            imwrite(outIm,strcat(outputFolder,filesep,positionList(positionIdx).name,filesep,'SegmentedImages',filesep,listSeg(2).name));             
            delete(strcat(outputFolder,filesep,positionList(positionIdx).name,filesep,'SegmentedImages',filesep,listSeg(1).name));
            delete(strcat(outputFolder,filesep,positionList(positionIdx).name,filesep,'SegmentedImages',filesep,listSeg(3).name));
        end
    end
end
