% ========================================================================= 
% rjBrowser_updateFrame, part of the trjbrowser suite for visualizing SPT
% trajectories. Copyright (C) 2015 Martin Lindén and Vladimir Curic
% E-mail: bmelinden@gmail.com, vladacuric@gmail.com
% =========================================================================
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or any
% later version.  This program is distributed in the hope that it will
% be useful, but WITHOUT ANY WARRANTY; without even the implied
% warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
% the GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program. If not, see <http://www.gnu.org/licenses/>.
% =========================================================================
%
% trj=trjBrowser_updateFrame(trj,frame)
% 
% internal trjBrowser function to update the trj movie to show frame
% frame. See help trjBrowser for usage.
%
% ML&VC, 2014-12-xx
function trj=trjBrowser_updateFrame(trj,frame)

%fprintf('movie %s changed to frame %d !\n',trj.movieName,frame);

trj.frameRadius=6; % number of forward and backward frames to show

% dummy change of frame

set(trj.imHandle,'Cdata',trj.mvdata(:,:,frame));

% find the trajectories to plot
trjInd=[];
for tt=1:length(trj.trj)
    if( trj.trj{tt}(1,3)<(frame+trj.frameRadius) && trj.trj{tt}(end,3)>(frame-trj.frameRadius) )
       trjInd(end+1)=tt; %#ok
    end
end
% delete existing plots
delete(trj.plotHandle)
% plot new ones and reset handles
figure(trj.mvFigure); % make the movie current axis
trj.plotHandle=[];

if(isfield(trj,'states'))
    for tt=1:length(trjInd)
        xyf=trj.trj{trjInd(tt)};
        s  =trj.states{trjInd(tt)};
        
        % plot active trajectories in the past, colored by state
        if( xyf(1,3)<=frame && xyf(end,3)>=frame )
            frameInd=find( xyf(:,3)<=frame);
            for ff=1:length(frameInd)-1
                trj.plotHandle(end+1,1)=plot(xyf(frameInd(ff:ff+1),1),xyf(frameInd(ff:ff+1),2),...
                    '-k','linew',1,'color',trj.stateColor(s(frameInd(ff)),:));
            end
            %trj.plotHandle(end+1,1)=plot(xyf(1,1),xyf(1,2),'ro','markersize',7);
            trj.plotHandle(end+1,1)=plot(xyf(frameInd(end),1),xyf(frameInd(end),2),'bo','markersize',7);
        end
    end
else
    for tt=1:length(trjInd)
        xyf=trj.trj{trjInd(tt)};
        
        %frameInd=find( (frame <=xyf(:,3)).*(xyf(:,3)<=frame+trj.frameRadius));
        %if(~isempty(frameInd))
        %    %trj.plotHandle(end+1,1)=plot(xyf(frameInd,1),xyf(frameInd,2),'-r','linewidth',0,'color',trj.trjFutureColor(trjInd(tt),:));
        %end
        
        % trj.frameRadiius in the past
        %frameInd=find( (frame-trj.frameRadius <= xyf(:,3)).*(xyf(:,3) <= frame));
        %if(~isempty(frameInd))
        %    trj.plotHandle(end+1,1)=plot(xyf(frameInd,1),xyf(frameInd,2),'-m.','linewidth',1,'color',trj.trjPastColor(trjInd(tt),:));
        %end
        
        % plot active trajectories in the past, and keep it for trj.frameRadius
        % after it ends
        if( xyf(1,3)<=frame && xyf(end,3)>=frame )
            frameInd=find( xyf(:,3)<=frame);
            trj.plotHandle(end+1,1)=plot(xyf(frameInd,1),xyf(frameInd,2),'-m','linewidth',2,'color',trj.trjPastColor(trjInd(tt),:));
            %%%trj.plotHandle(end+1,1)=plot(xyf(1,1),xyf(1,2),'*','markersize',7,'color',trj.trjPastColor(trjInd(tt),:));
            trj.plotHandle(end+1,1)=plot(xyf(1,1),xyf(1,2),'ro','markersize',7);
            trj.plotHandle(end+1,1)=plot(xyf(frameInd(end),1),xyf(frameInd(end),2),'bo','markersize',7);
            
       
        end
        
        
        frameInd=find(xyf(:,3)==frame,1);
        if(~isempty(frameInd))
            %%%trj.plotHandle(end+1,1)=plot(xyf(frameInd,1),xyf(frameInd,2),'b.',...
            %%%    'markerface',trj.trjFutureColor(trjInd(tt),:),'markeredge',trj.trjPastColor(trjInd(tt),:));
        end
    end
end
        title(['frame ' int2str(frame)])
