function  [bestParams,bestResidual] = fit1To1Model(assocSignal,assocTime,dissocSignal,dissocTime,protconc,numGuesses,guessMin,guessMax,lockRmax)
%This function fits a 1to1 exponential model to assocatio and dissocation SPR data from a protein titration
%params=[ka,kd,Rmax,t0,R0_1,...,R0_n,...,Rinfinity_1,...R0_n]
%t0 is t shift for assocation. R0 is inital value for dissocation curves.
%Rinfinity is offset of dissoctio curves
numCurves=numel(protconc);
funDiff=@(x) diff1To1Model(x,assocSignal,assocTime,dissocSignal,dissocTime,protconc,numCurves);

numParams=4+numCurves*2;

lb=zeros(1,numParams);
ub=inf*ones(1,numParams);
lb(4)=-1;
ub(4)=1;
guessCenter=mean([guessMax;guessMin]);

%lb(5:numCurves+5-1)=0.95*guessCenter(5:numCurves+5-1); %R0
%ub(5:numCurves+5-1)=1.05*guessCenter(5:numCurves+5-1);
lb(end-numCurves+1:end)=0;%Rinfinity
ub(end-numCurves+1:end)=1e-5;

if lockRmax==1
    
   lb(3)=mean([guessMax(3) guessMin(3)]); 
   ub(3)=lb(3)*1.00001;
    
end

options=optimoptions('fmincon','Display','off','TolX',1e-5,'TolFun',1e-5);
paramsAll=zeros(numGuesses,numParams);
resAll=zeros(numGuesses,1);
parfor i= 1:numGuesses
    
    %params0=10.^(guessMinLog10+rand(1,numel(lb)).*(guessMaxLog10-guessMinLog10));
    params0=guessMin+(guessMax-guessMin).*rand(1,numParams);
    
    
    [paramsCurr,res] = fmincon(funDiff,params0,[],[],[],[],lb,ub,[],options);
    resAll(i)=res;
    paramsAll(i,:)=paramsCurr;
    
    
end

[bestResidual,bestind]=min(resAll);
bestParams=paramsAll(bestind,:);





end

