function  [bestParams,bestResidual] = fit1To1ModelLockR0Manyka(assocSignal,assocTime,dissocSignal,dissocTime,protconc,numGuesses,guessMin,guessMax,lockRmax)
%This function fits a 1to1 exponential model to assocatio and dissocation SPR data from a protein titration
%params=[ka_1,...ka_n,kd,Rmax,t0,R0_1,...,R0_n,...,Rinfinity_1,...R0_n]
%t0 is t shift for assocation. R0 is inital value for dissocation curves.
%Rinfinity is offset of dissoctio curves
numCurves=numel(protconc);
funDiff=@(x) diff1To1ModelLockR0Manyka(x,assocSignal,assocTime,dissocSignal,dissocTime,protconc,numCurves);

numParams=numCurves+3+numCurves*2;

lb=zeros(1,numParams);
ub=inf*ones(1,numParams);

lb(1:numCurves)=guessMin(1:numCurves)/10;
ub(1:numCurves)=guessMax(1:numCurves)*10;

%lb(numCurves+2)=guessMin(numCurves+2);
%ub(numCurves+2)=guessMax(numCurves+2);

lb(numCurves+3)=-1;
ub(numCurves+3)=1;
%guessCenter=mean([guessMax;guessMin]);

lb(numCurves+4:numCurves+4+numCurves-1)=0; %R0
ub(numCurves+4:numCurves+4+numCurves-1)=1e-5;
lb(end-numCurves+1:end)=0;%Rinfinity
%ub(end-numCurves+1:end)=guessMax(numCurves+2);
ub(end-numCurves+1:end)=1e-5;
if lockRmax==1
    
   lb(numCurves+2)=mean([guessMax(numCurves+2) guessMin(numCurves+2)]); 
   ub(numCurves+2)=lb(numCurves+2)*1.00001;
    
end

options=optimoptions('fmincon','Display','off','TolX',1e-5,'TolFun',1e-5);
paramsAll=zeros(numGuesses,numParams);
resAll=zeros(numGuesses,1);
parfor i= 1:numGuesses
    
    %params0=10.^(guessMinLog10+rand(1,numel(lb)).*(guessMaxLog10-guessMinLog10));
    params0=guessMin+(guessMax-guessMin).*rand(1,numParams);
    
    
    [paramsCurr,res] = fmincon(funDiff,params0,[],[],[],[],lb,ub,[],options);
    try
     resAll(i)=res;
    catch
       dsd=1; 
       
    end
        
    paramsAll(i,:)=paramsCurr;
    
    
end

[bestResidual,bestind]=min(resAll);
bestParams=paramsAll(bestind,:);





end

