%ver3: Lock R0

addpath(genpath('../Functions/'));

datafolder='../../../data_spr/';

%%
numEndPointsToSkip=10;
timeStep=0.1; %s
slopeFrames=1:20;
numGuessExponential=12;
maxRate = 0.5;%In the exponential fitting ,when fitted ka of a specific curve * protein is largar than this value, protein concentration with ka*[LacI] larger then this is not used in the ka estimate. For example if ka*[LacI] = 1 s^-1, all DNAs get bound in approx 1 s, => not suitable to use this concentration, curve gets saturated to fast

%%
listingExperiments=dir([datafolder 'Exp*']);

numExps=numel(listingExperiments);

numMeasPerExp=zeros(1,numExps);
experiments=cell(1,numExps);

for i=1:numExps
    listingMeas=dir([datafolder listingExperiments(i).name '/*.txt']);
    
    listingMeas=natsortfiles(listingMeas);
    
    numMeas=numel(listingMeas);
    numMeasPerExp(i)=numMeas;
    experiments{i}.meas=cell(1,numMeas);
    for j=1:numMeas
        currPath = [listingMeas(j).folder '/' listingMeas(j).name];
        experiments{i}.meas{j}.filename=listingMeas(j).name;
        experiments{i}.meas{j}.folder=listingMeas(j).folder;
        disp(['Loading data from file: ' currPath]);
        
        splitFilename=strsplit(experiments{i}.meas{j}.filename,'_');
        experiments{i}.meas{j}.operator=splitFilename{2};
        concstr1=splitFilename{3};
        concstr2=strsplit(concstr1,'.');
        experiments{i}.meas{j}.saltconc=str2double(concstr2{1});
        
        
        fid = fopen(currPath);
        header=fgetl(fid);
        header=strrep(header,',','.');
        header=strsplit(header,'\t');
        
         
        
        experiments{i}.meas{j}.header=header;
        
        numcols=numel(header);
        formatSpec=[];
        for k=1:numcols
           formatSpec=[formatSpec '%s ']; 
        end
        experiments{i}.meas{j}.textdata=textscan(fid,formatSpec,'Delimiter','\t');
        fclose(fid);
        
        numLines=numel(experiments{i}.meas{j}.textdata{1})-numEndPointsToSkip;
        
        data=zeros(numLines,numcols);
        conc=zeros(1,numcols);
        for k=1:numcols
           headersplit=strsplit(header{k},' '); 
           conc(k)=str2double(headersplit{4});
     
           for index=1:numLines 
               
               thetext=experiments{i}.meas{j}.textdata{k}{index};
               thetext=strrep(thetext,',','.');
               data(index,k)=str2num(thetext);
           end
           
           experiments{i}.meas{j}.time=data(:,1:2:end-1);
           experiments{i}.meas{j}.signal=data(:,2:2:end);
           experiments{i}.meas{j}.protconc=conc(1:2:end-1);
        end
        
        %experiments{i}.meas{j}= importdata(currPath,'\t');
        
    end
end
%%
RmaxTheory{1}=NaN*ones(1,numMeas);
RmaxTheory{2}=NaN*ones(1,numMeas);
RmaxTheory{3}=[128 128 128 128 120 120 120 120 NaN NaN NaN NaN];
RmaxTheory{4}=[160 160 160 160 134 134 134 134 NaN NaN NaN NaN];
RmaxTheory{5}=[NaN NaN NaN NaN 71 71 71 71 73 73 73 73 ];
RmaxTheory{6}=[NaN NaN NaN NaN 65 65 65 65 65 65 65 65];

%%

assocStart{1}=1.5;%Part of the curve (in seconds) that is the association experiment
assocEnd{1}=119;
dissocStart{1}=[121];


for i=2:numExps
   assocStart{i}=assocStart{1};
   assocEnd{i}=assocEnd{1};
   dissocStart{i}=dissocStart{1};
end
%% Example curves

expInd=6;
measInd=8;

figure();
plot(experiments{expInd}.meas{measInd}.time,experiments{expInd}.meas{measInd}.signal);

%%
for i=1:numExps
    for j=1:numMeas
        numCurves=numel(experiments{i}.meas{j}.protconc);
        experiments{i}.meas{j}.assocSignal=cell(1,numCurves);
        experiments{i}.meas{j}.dissocSignal=cell(1,numCurves);
        experiments{i}.meas{j}.assocTime=cell(1,numCurves);
        experiments{i}.meas{j}.dissocTime=cell(1,numCurves);
        
        
        for k=1:numCurves 
            time=experiments{i}.meas{j}.time(:,k);
            [~,startAssocIndex]=min(abs(assocStart{i}-time));
            [~,endAssocIndex]=min(abs(assocEnd{i}-time));
            [~,startDissocIndex]=min(abs(dissocStart{i}-time));
            
            experiments{i}.meas{j}.assocSignal{k}=experiments{i}.meas{j}.signal(startAssocIndex:endAssocIndex,k);
            experiments{i}.meas{j}.assocTime{k}=time(startAssocIndex:endAssocIndex);
            
            experiments{i}.meas{j}.dissocSignal{k}=experiments{i}.meas{j}.signal(startDissocIndex:end,k);
            experiments{i}.meas{j}.dissocTime{k}=time(startDissocIndex:end)-time(startDissocIndex);
            experiments{i}.meas{j}.dissocTimeAbsolute{k}=time(startDissocIndex:end);
        
        end
    end
end

%% Example curves

expInd=5;
measInd=9;
curveInd=4;

figure();
plot(experiments{expInd}.meas{measInd}.assocTime{curveInd},experiments{expInd}.meas{measInd}.assocSignal{curveInd});
figure();
plot(experiments{expInd}.meas{measInd}.dissocTime{curveInd},experiments{expInd}.meas{measInd}.dissocSignal{curveInd});

%% Get Rmax
%Simples possible assumption, final value in the association curve of the
%lowest salt concentration highest protein of this operator


for i=1:numExps
   for j=1:numMeas
      numCurves=numel(experiments{i}.meas{j}.protconc);
      
      experiments{i}.meas{j}.RmaxSimple=[];
      index=floor((j-1)/4)*4+1; %1 5 or 9
      experiments{i}.meas{j}.RmaxSimple=experiments{i}.meas{index}.assocSignal{end}(end);
     
   end
end
%% Calculate inital slopes, for ka and kd estimates

kaSlopeVals=zeros(numExps,numMeas,numCurves);
kaSlopeValsRmaxSimple=zeros(numExps,numMeas,numCurves);
kdSlopeVals=zeros(numExps,numMeas,numCurves);

for i=1:numExps
    for j=1:numMeas
        numCurves=numel(experiments{i}.meas{j}.protconc);
        for k=1:numCurves 
            %normAssocSignal=experiments{i}.meas{j}.assocSignal{k}/experiments{i}.meas{j}.RmaxSimple;
            %normAssocSignal=experiments{i}.meas{j}.assocSignal{k}/RmaxTheory{i}(j);
            normDissocSignal=experiments{i}.meas{j}.dissocSignal{k}/experiments{i}.meas{j}.dissocSignal{k}(1);
            
            p=polyfit(experiments{i}.meas{j}.assocTime{k}(slopeFrames),experiments{i}.meas{j}.assocSignal{k}(slopeFrames),1);
            slope=p(1);
            kaSlope=slope/(experiments{i}.meas{j}.protconc(k)*experiments{i}.meas{j}.RmaxSimple);
            kaSlopeValsRmaxSimple(i,j,k)=kaSlope;
            kaSlope=slope/(experiments{i}.meas{j}.protconc(k)*RmaxTheory{i}(j));
            kaSlopeVals(i,j,k)=kaSlope;
            
            
            kdSlope=-polyfit(experiments{i}.meas{j}.dissocTime{k}(slopeFrames),normDissocSignal(slopeFrames),1);
            kdSlope=kdSlope(1);
            kdSlopeVals(i,j,k)=kdSlope;
        
        end
    end
end
%% plot ka kd slope method
%expInd1=4;
expInd2=5;
expInd3=6;
measVec{1}=1:4;
measVec{2}=[5:8];
measVec{3}=[5:8];
curveInd1=10;
curveInd2=5;
curveInd3=5;

figure();
hold on;
%plot(kdSlopeVals(expInd1,measVec{1},curveInd1),kaSlopeVals(expInd1,measVec{1},curveInd1),'bx');
plot(kdSlopeVals(expInd2,measVec{2},curveInd2),kaSlopeVals(expInd2,measVec{2},curveInd2),'rx');
plot(kdSlopeVals(expInd3,measVec{3},curveInd3),kaSlopeVals(expInd3,measVec{3},curveInd3),'mx');

%xLim=xlim();
%yLim=ylim();
%xlim([0 xLim(2)*1.5]);
%ylim([0 yLim(2)*1.5]);
xlim([0 0.05]);
ylim([0 0.018]);

xlabel('kd (s^-^1)');
ylabel('ka s^-^1 nM^-^1')

title(['Fig. S3B Slope method, curveInds :' num2str(curveInd2) ', ' num2str(curveInd3)]);

savefig('ka_vs_kd_SPR_slope')
%% average slope results

%expInds=[4 5];
%curveInds=[10 4];
%measInds{1}=[1:4];
%measInds{2}=[5:8];

%expInds=[4 5 6];
%curveInds=[10 4 4];
%measInds{1}=[1:4];
%measInds{2}=[5:8];
%measInds{3}=[5:8];

expInds=[5 6];
curveInds=[4 4];
measInds{1}=[5:8];
measInds{2}=[5:8];

numRepeats=numel(expInds);
numSalts=numel(measInds{1});

kaSamples=zeros(numRepeats,numSalts);
kdSamples=zeros(numRepeats,numSalts);

for i=1:numRepeats
   kaSamples(i,:)= kaSlopeVals(expInds(i),measInds{i},curveInds(i));
   kdSamples(i,:)= kdSlopeVals(expInds(i),measInds{i},curveInds(i));
    
end

kaAverages=mean(kaSamples);
kdAverages=mean(kdSamples);

kaErrors=std(kaSamples)/sqrt(numRepeats);
kdErrors=std(kdSamples)/sqrt(numRepeats);

figure;
errorbar(kdAverages,kaAverages,kaErrors,kaErrors,kdErrors,kdErrors,'bx');
xlabel('k_d (s^-^1)');
ylabel('k_a (s^-^1 nM^-^1)');
%ylim([0 8]*1e-3);
%xlim([0 0.035]);
xlim([0 0.05]);
ylim([0 0.017]);

kaNorm=kaSamples./repmat(kaSamples(:,1),[1 numSalts]);
kaNormAverages=mean(kaNorm);
kaNormErrors=std(kaNorm)/sqrt(numRepeats);

figure;
errorbar(kdAverages,kaNormAverages,kaNormErrors,kaNormErrors,kdErrors,kdErrors,'bx');
xlabel('k_d (s^-^1)');
ylabel('k_a/k_a(150 mM NaCl)');
ylim([0 1.2]);
%xlim([0 0.035]);
xlim([0 0.05]);




%% Fit to exponential model

%params=[ka,kd,Rmax,t0,R0_1,...,R0_n,...,Rinfinity_1,...R0_n]
protConcUsed{1}=[2.5 5 10];%For the fitting, concentrations closest to these cocnentrations are used
protConcUsed{2}=[2.5 5 10];
protConcUsed{3}=[2.5 5 10];
protConcUsed{4}=[2.5 5 10];
protConcUsed{5}=[1 2 3.9000 7.8000 15.6000 31.2500 62.5000 125 250 500];
protConcUsed{6}=[1 2 3.9000 7.8000 15.6000 31.2500 62.5000 125 250 500];
slopeFromInd=[3 3 3 3 5 5]; %For the start guess for ka and kd
%protConcUsed{1}=[10];%For the fitting, concentrations closest to these cocnentrations are used
%protConcUsed{2}=[10];
%protConcUsed{3}=[10];
%protConcUsed{4}=[10];
%protConcUsed{5}=[500];
%numCurvesUsed=1;




kaExpFitVals=zeros(numExps,numMeas);
kdExpFitVals=zeros(numExps,numMeas);

for i=1:numExps
    
    numCurvesUsed=numel(protConcUsed{i});
    numParams=numCurvesUsed+3+numCurvesUsed*2;
    lockRmax=0;
    for j=1:numMeas
        experiments{i}.meas{j}.expfitInitial=[];
        curvesUsed=zeros(1,numCurvesUsed);
        for k=1:numCurvesUsed
             [~,curvesUsed(k)]=min(abs(experiments{i}.meas{j}.protconc-protConcUsed{i}(k)));
        end
        
       
%         numPointsAssoc=numel(experiments{i}.meas{j}.assocSignal{1});
%         numPointsDissoc=numel(experiments{i}.meas{j}.dissocSignal{1});
%         
%         assocSignalMatr=zeros(numPointsAssoc,numCurves);
%         dissocSignalMatr=zeros(numPointsDissoc,numCurves);
%         
%         assocTimeMatr=zeros(numPointsAssoc,numCurves);
%         dissocTimeMatr=zeros(numPointsDissoc,numCurves);
%         for k=1:numCurves 
%             
%             assocSignalMatr(:,k)=experiments{i}.meas{j}.assocSignal{k};
%             dissocSignalMatr(:,k)=experiments{i}.meas{j}.dissocSignal{k};
%             
%             assocTimeMatr(:,k)=experiments{i}.meas{j}.assocTime{k};
%             dissocTimeMatr(:,k)=experiments{i}.meas{j}.dissocTime{k};
%         
%         end


         assocSignalUsed=cell(1,numCurvesUsed);
         dissocSignalUsed=cell(1,numCurvesUsed);
         
         assocTimeUsed=cell(1,numCurvesUsed);
         dissocTimeUsed=cell(1,numCurvesUsed);
         
         protconcUsed=zeros(1,numCurvesUsed);
         
         
         for k=1:numCurvesUsed
            assocSignalUsed{k}= experiments{i}.meas{j}.assocSignal{curvesUsed(k)};
            dissocSignalUsed{k}= experiments{i}.meas{j}.dissocSignal{curvesUsed(k)};
            
            assocTimeUsed{k}= experiments{i}.meas{j}.assocTime{curvesUsed(k)};
            dissocTimeUsed{k}= experiments{i}.meas{j}.dissocTime{curvesUsed(k)};
            protconcUsed(k)=experiments{i}.meas{j}.protconc(curvesUsed(k));
            
            
         end
         
         guessCenter=zeros(1,numParams);
         guessCenter(1:numCurvesUsed)=kaSlopeVals(i,j,curvesUsed(slopeFromInd(i)));
         guessCenter(numCurvesUsed+1)=kdSlopeVals(i,j,curvesUsed(slopeFromInd(i)));
         %guessCenter(3)=experiments{i}.meas{j}.RmaxSimple;
         guessCenter(numCurvesUsed+2)=RmaxTheory{i}(j);
         guessCenter(numCurvesUsed+3)=0;
         
         dissocStarts=zeros(1,numCurvesUsed);
         
         for k=1:numCurvesUsed
            dissocStarts(k)=dissocSignalUsed{k}(1);  
         end
         
         guessCenter(numCurvesUsed+4:numCurvesUsed+4+numCurvesUsed-1)=dissocStarts;
         %guessCenter(end-numCurvesUsed+1:end)=0;
         guessCenter(end-numCurvesUsed+1:end)=guessCenter(numCurvesUsed+2)/10;
         
         guessMin=guessCenter*0.5;
         guessMax=guessCenter*2;
         
         if sum(isnan(guessCenter))==0
             
             [params,residual]=fit1To1ModelLockR0Manyka(assocSignalUsed,assocTimeUsed,dissocSignalUsed,dissocTimeUsed,protconcUsed,numGuessExponential,guessMin,guessMax,lockRmax);
             experiments{i}.meas{j}.expfitInitial.params=params;
             experiments{i}.meas{j}.expfitInitial.residual=residual;
             experiments{i}.meas{j}.expfitInitial.guessCenter=guessCenter;
             experiments{i}.meas{j}.expfitInitial.curvesUsed=curvesUsed;
             kdExpFitVals(i,j)=params(numCurvesUsed+1);
             % kaExpFitVals(i,j)=mean(params(1:numCurvesUsed));
             rateVals= protconcUsed.*params(1:numCurvesUsed);
             concToFit=protconcUsed;
             maxInd=find(rateVals>maxRate,1);
             if ~isempty(maxInd)
                 rateVals=rateVals(1:maxInd-1);
                 concToFit=protconcUsed(1:maxInd-1);
             end
          
             
             p=polyfit(concToFit,rateVals,1);
             kaExpFitVals(i,j)=p(1);
             
           
        end
    end
end

%% Show fit

expInd=5;
measInd=6;
usedInd=4;
curveInd=experiments{expInd}.meas{measInd}.expfitInitial.curvesUsed(usedInd);
 numCurvesUsed=numel(protConcUsed{expInd});




params=experiments{expInd}.meas{measInd}.expfitInitial.params;
protconccurr=experiments{expInd}.meas{measInd}.protconc(curveInd);
%R0s=params(5:5+numCurvesUsed-1);
Rinfinities=params(end-numCurvesUsed+1:end);

assocTime=experiments{expInd}.meas{measInd}.assocTime{curveInd};
dissocTime=experiments{expInd}.meas{measInd}.dissocTime{curveInd};
assocSignal=experiments{expInd}.meas{measInd}.assocSignal{curveInd};
dissocSignal=experiments{expInd}.meas{measInd}.dissocSignal{curveInd};

assocModel=eval1to1ModelAssoc([params(usedInd) params(numCurvesUsed+1:numCurvesUsed+3)],protconccurr,assocTime);
R0=assocModel(end)-Rinfinities(curveInd);
dissocModel=eval1to1ModelDissoc([params(numCurvesUsed+1) R0 Rinfinities(usedInd)],dissocTime);

figure();

subplot(1,2,1);
plot(assocTime,assocSignal,'b');
hold on;
plot(assocTime,assocModel,'r--');
title('Association');

subplot(1,2,2);
plot(dissocTime,dissocSignal,'b');
hold on;
plot(dissocTime,dissocModel,'r--');
title('Dissociation');




%% plot ka kd exponential 1to1 fit method

expInd=5;
measVec{1}=1:4;
measVec{2}=5:8;
measVec{3}=9:12;


figure();
hold on;
plot(kdExpFitVals(expInd,measVec{1}),kaExpFitVals(expInd,measVec{1}),'bo');
plot(kdExpFitVals(expInd,measVec{2}),kaExpFitVals(expInd,measVec{2}),'ro');
plot(kdExpFitVals(expInd,measVec{3}),kaExpFitVals(expInd,measVec{3}),'mo');
title('Exponential fit individual Rmax fit')
xLim=xlim();
yLim=ylim();
xlim([0 xLim(2)*1.5]);
ylim([0 yLim(2)*1.5]);

xlabel('kd (s^-^1)');
ylabel('ka s^-^1 nM^-^1')


%% Set Rmax to same value for all measuremtns in a salt titration, get value from lowest salt fitted value
%


for i=1:numExps
    for j=1:numMeas
        numCurves=numel(experiments{i}.meas{j}.protconc);
        numCurvesUsed=numel(protConcUsed{i});
        experiments{i}.meas{j}.RmaxFitFromLowSalt=[];
        index=floor((j-1)/4)*4+1; %1 5 or 9
        if ~isempty(experiments{i}.meas{index}.expfitInitial)
            experiments{i}.meas{j}.RmaxFitFromLowSalt=experiments{i}.meas{index}.expfitInitial.params(numCurvesUsed+2);
        end
    end
end

%% Fit to exponential model again, but now lock the Rmax

%params=[ka,kd,Rmax,t0,R0_1,...,R0_n,...,Rinfinity_1,...R0_n]
protConcUsed{1}=[2.5 5 10];%For the fitting, concentrations closest to these cocnentrations are used
protConcUsed{2}=[2.5 5 10];
protConcUsed{3}=[2.5 5 10];
protConcUsed{4}=[2.5 5 10];
protConcUsed{5}=[1 2 3.9000 7.8000 15.6000 31.2500 62.5000 125 250 500];
protConcUsed{6}=[1 2 3.9000 7.8000 15.6000 31.2500 62.5000 125 250 500];



%protConcUsed{1}=[10];%For the fitting, concentrations closest to these cocnentrations are used
%protConcUsed{2}=[10];
%protConcUsed{3}=[10];
%protConcUsed{4}=[10];
%protConcUsed{5}=[10];
%numCurvesUsed=1;




kaRmaxLockedVals=zeros(numExps,numMeas);
kdRmaxLockedVals=zeros(numExps,numMeas);

for i=1:numExps
    numCurvesUsed=numel(protConcUsed{i});
    numParams=numCurvesUsed+3+numCurvesUsed*2;
    lockRmax=1;
    for j=1:numMeas
        
        if ~isempty(experiments{i}.meas{j}.expfitInitial)
            curvesUsed=zeros(1,numCurvesUsed);
            for k=1:numCurvesUsed
                [~,curvesUsed(k)]=min(abs(experiments{i}.meas{j}.protconc-protConcUsed{i}(k)));
            end
            
            
            %         numPointsAssoc=numel(experiments{i}.meas{j}.assocSignal{1});
            %         numPointsDissoc=numel(experiments{i}.meas{j}.dissocSignal{1});
            %
            %         assocSignalMatr=zeros(numPointsAssoc,numCurves);
            %         dissocSignalMatr=zeros(numPointsDissoc,numCurves);
            %
            %         assocTimeMatr=zeros(numPointsAssoc,numCurves);
            %         dissocTimeMatr=zeros(numPointsDissoc,numCurves);
            %         for k=1:numCurves
            %
            %             assocSignalMatr(:,k)=experiments{i}.meas{j}.assocSignal{k};
            %             dissocSignalMatr(:,k)=experiments{i}.meas{j}.dissocSignal{k};
            %
            %             assocTimeMatr(:,k)=experiments{i}.meas{j}.assocTime{k};
            %             dissocTimeMatr(:,k)=experiments{i}.meas{j}.dissocTime{k};
            %
            %         end
            
            
            assocSignalUsed=cell(1,numCurvesUsed);
            dissocSignalUsed=cell(1,numCurvesUsed);
            
            assocTimeUsed=cell(1,numCurvesUsed);
            dissocTimeUsed=cell(1,numCurvesUsed);
            
            protconcUsed=zeros(1,numCurvesUsed);
            
            
            for k=1:numCurvesUsed
                assocSignalUsed{k}= experiments{i}.meas{j}.assocSignal{curvesUsed(k)};
                dissocSignalUsed{k}= experiments{i}.meas{j}.dissocSignal{curvesUsed(k)};
                
                assocTimeUsed{k}= experiments{i}.meas{j}.assocTime{curvesUsed(k)};
                dissocTimeUsed{k}= experiments{i}.meas{j}.dissocTime{curvesUsed(k)};
                protconcUsed(k)=experiments{i}.meas{j}.protconc(curvesUsed(k));
                
                
            end
            
            guessCenter=zeros(1,numParams);
            guessCenter(1:numCurvesUsed)=kaSlopeVals(i,j,curvesUsed(slopeFromInd(i)));
            guessCenter(numCurvesUsed+1)=kdSlopeVals(i,j,curvesUsed(slopeFromInd(i)));
            %guessCenter(3)=experiments{i}.meas{j}.RmaxSimple;
            %guessCenter(numCurvesUsed+2)=RmaxTheory{i}(j);
            guessCenter(numCurvesUsed+2)=experiments{i}.meas{j}.RmaxFitFromLowSalt;
            guessCenter(numCurvesUsed+3)=0;
            
            dissocStarts=zeros(1,numCurvesUsed);
            
            for k=1:numCurvesUsed
                dissocStarts(k)=dissocSignalUsed{k}(1);
            end
            
            guessCenter(numCurvesUsed+4:numCurvesUsed+4+numCurvesUsed-1)=dissocStarts;
            %guessCenter(end-numCurvesUsed+1:end)=0;
            guessCenter(end-numCurvesUsed+1:end)=guessCenter(numCurvesUsed+2)/10;
         
            
            guessMin=guessCenter*0.5;
            guessMax=guessCenter*2;
            
            guessMin(numCurvesUsed+2)=experiments{i}.meas{j}.RmaxFitFromLowSalt;
            guessMax(numCurvesUsed+2)=experiments{i}.meas{j}.RmaxFitFromLowSalt;
            
            if sum(isnan(guessCenter))==0
                
                [params,residual]=fit1To1ModelLockR0Manyka(assocSignalUsed,assocTimeUsed,dissocSignalUsed,dissocTimeUsed,protconcUsed,numGuessExponential,guessMin,guessMax,lockRmax);
                experiments{i}.meas{j}.expfitRmaxLocked.params=params;
                experiments{i}.meas{j}.expfitRmaxLocked.residual=residual;
                experiments{i}.meas{j}.expfitRmaxLocked.guessCenter=guessCenter;
                experiments{i}.meas{j}.expfitRmaxLocked.curvesUsed=curvesUsed;
                
                kdRmaxLockedVals(i,j)=params(numCurvesUsed+1);
                
                rateVals= protconcUsed.*params(1:numCurvesUsed);
                concToFit=protconcUsed;
                maxInd=find(rateVals>maxRate,1);
                if ~isempty(maxInd)
                    rateVals=rateVals(1:maxInd-1);
                    concToFit=protconcUsed(1:maxInd-1);
                end
                
                
                p=polyfit(concToFit,rateVals,1);
                kaRmaxLockedVals(i,j)=p(1);
                
            end
        end
    end
end



%% Show fit

expInd=6;
measInd=8;
usedInd=5;
 numCurvesUsed=numel(protConcUsed{expInd});
curveInd=experiments{expInd}.meas{measInd}.expfitRmaxLocked.curvesUsed(usedInd);





params=experiments{expInd}.meas{measInd}.expfitRmaxLocked.params;
protconccurr=experiments{expInd}.meas{measInd}.protconc(curveInd);
R0s=params(5:5+numCurvesUsed-1);
Rinfinities=params(end-numCurvesUsed+1:end);

assocTime=experiments{expInd}.meas{measInd}.assocTime{curveInd};
dissocTime=experiments{expInd}.meas{measInd}.dissocTime{curveInd};
assocSignal=experiments{expInd}.meas{measInd}.assocSignal{curveInd};
dissocSignal=experiments{expInd}.meas{measInd}.dissocSignal{curveInd};

assocModel=eval1to1ModelAssoc([params(usedInd) params(numCurvesUsed+1:numCurvesUsed+3)],protconccurr,assocTime);
R0=assocModel(end)-Rinfinities(curveInd);
dissocModel=eval1to1ModelDissoc([params(numCurvesUsed+1) R0 Rinfinities(usedInd)],dissocTime);


figure();

subplot(1,2,1);
plot(assocTime,assocSignal,'b');
hold on;
plot(assocTime,assocModel,'r--');
title('Association Rmaxlocked');

subplot(1,2,2);
plot(dissocTime,dissocSignal,'b');
hold on;
plot(dissocTime,dissocModel,'r--');
title('Dissociation');

%% plot ka kd exponential 1to1 fit method locked Rmax

%expInd1=4;
expInd2=5;
expInd3=6;
%measVec{1}=1:4;
measVec{2}= [5:8];
measVec{3}=[5:8];


figure();
hold on;
%plot(kdRmaxLockedVals(expInd1,measVec{1}),kaRmaxLockedVals(expInd1,measVec{1}),'bo');
plot(kdRmaxLockedVals(expInd2,measVec{2}),kaRmaxLockedVals(expInd2,measVec{2}),'rx');
plot(kdRmaxLockedVals(expInd3,measVec{3}),kaRmaxLockedVals(expInd3,measVec{3}),'mx');

%xLim=xlim();
%yLim=ylim();
%xlim([0 xLim(2)*1.5]);
%ylim([0 yLim(2)*1.5]);
xlim([0 0.05]);
ylim([0 0.018]);

xlabel('kd (s^-^1)');
ylabel('ka s^-^1 nM^-^1')
title(['Fig. s3B Exponential fit Rmax from low salt']);

savefig('ka_vs_kd_SPR_fit')

%% Eestimate variation in koffmicro ptot, from the two repats with O1
varRatio=[];
kdValsSelected{1}=kdRmaxLockedVals(expInd2,measVec{2});
kdValsSelected{2}=kdRmaxLockedVals(expInd3,measVec{3});
kaValsSelected{1}=kaRmaxLockedVals(expInd2,measVec{2});
kaValsSelected{2}=kaRmaxLockedVals(expInd3,measVec{3});

for i=1:2
    
    normX=std(kdValsSelected{i});
    normY=std(kaValsSelected{i});
    relX=kdValsSelected{i}/normX;
    relY=kaValsSelected{i}/normY;
    
    bestParams=fitLineErrorxyver3(relX,relY,10,0.5,1.5,1e-3,1e-3);
    
    
    pfit{i} = [bestParams(1)*normY/normX bestParams(2)*normY];
    
    
    numcurrvals=numel(kdValsSelected{i});
    koffmicrovals=pfit{i}(2).*kdValsSelected{i}./(pfit{i}(2)-kaValsSelected{i});
    ptotvals=kaValsSelected{i}./pfit{i}(2);
    
    okkas=kaValsSelected{i}<0.95*pfit{i}(2); %Only stiamte koffmicro and ptot for point which are samller than 95% of konmax
    ptotvals=ptotvals(okkas);
    koffmicrovals=koffmicrovals(okkas);
    
    
    ptotVar=std(ptotvals)/mean(ptotvals);
    koffmicroVar=std(koffmicrovals)/mean(koffmicrovals);
    
    varRatio(i)=ptotVar/koffmicroVar;
    
end


disp(['Ratio in variance in ptot/koffmicro is : ' num2str(mean(varRatio)) '+-' num2str(std(varRatio)/sqrt(2)) ]);

%% Sensograms for whole measurement

expInd=6;
for measInd=5:8
    signals=experiments{expInd}.meas{measInd}.signal(:,experiments{expInd}.meas{measInd}.expfitRmaxLocked.curvesUsed);
    times=experiments{expInd}.meas{measInd}.time(:,experiments{expInd}.meas{measInd}.expfitRmaxLocked.curvesUsed);
    
    figure();
    plot(times,signals,'LineWidth',3);
    xlim([0 max(max(times))]);
    ylim([0 1.2*max(max(signals))]);
    
    hold on;
    usedInd=1;
    for curveInd=experiments{expInd}.meas{measInd}.expfitRmaxLocked.curvesUsed
        
        
        
        
        
        params=experiments{expInd}.meas{measInd}.expfitRmaxLocked.params;
        protconccurr=experiments{expInd}.meas{measInd}.protconc(curveInd);
        R0s=params(5:5+numCurvesUsed-1);
        
        
        Rinfinities=params(end-numCurvesUsed+1:end);
        
        assocTime=experiments{expInd}.meas{measInd}.assocTime{curveInd};
        dissocTime=experiments{expInd}.meas{measInd}.dissocTime{curveInd};
        assocSignal=experiments{expInd}.meas{measInd}.assocSignal{curveInd};
        dissocSignal=experiments{expInd}.meas{measInd}.dissocSignal{curveInd};
        
        assocModel=eval1to1ModelAssoc([params(usedInd) params(numCurvesUsed+1:numCurvesUsed+3)],protconccurr,assocTime);
        R0=assocModel(end)-Rinfinities(curveInd);
        dissocModel=eval1to1ModelDissoc([params(numCurvesUsed+1) R0 Rinfinities(usedInd)],dissocTime);
        
        dissocTimeAbsolute=experiments{expInd}.meas{measInd}.dissocTimeAbsolute{curveInd};
        
        plot(assocTime,assocModel,'r--','LineWidth',1);
        plot(dissocTimeAbsolute,dissocModel,'r--','LineWidth',1);
        usedInd=usedInd+1;
        
        xlabel('Time (s)');
        ylabel('Response (RU)');
        
  
        
    end
    ylim([0 90]);
    xlim([0 450]);
    title('Fig. S3A')
    savefig(['Sensorgrams_exp ' num2str(expInd) ' meas ' num2str(measInd)]);
end
