function [trajectories,dnaCoords] = runOnOffSimulation(kdVec,kaVec,numSitesVec,xDim,yDim,names,T_dt,tmax,equillibrated)
%This function runs an binding/unbinding experimen, returns trajectores and
%reactions suitible for export to be ued as import for SMeagol to do
%simulated microscopy

%time inbetween two time-points at which the simulation state is saved
%T_dt = 1/(60*60);
% Column vector of time points at which the simulation state is saved
T = (0:T_dt:tmax)';
tspan= [T(1) T(end)];

numSiteTypes=numel(numSitesVec);
totNumSites=sum(numSitesVec);
% Matrix defining the change in copy numer for each reaction. See function
% for a more in-depth description



S =zeros(2*numSiteTypes,numSiteTypes);
k=1;
for i=1:totNumSites
   if mod(i,2)==1 
   S(i,ceil(i/2))=-1; %Dissocation
   
   else
   S(i,i/2)=1; %Assocation
   
   end
end
S=sparse(S);

kdVecAll=zeros(1,totNumSites);
kaVecAll=zeros(1,totNumSites);

k=1;

for i=1:numSiteTypes
    kdVecAll(k:k+numSitesVec(i)-1)=kdVec(i);
    kaVecAll(k:k+numSitesVec(i)-1)=kaVec(i);
    k=k+numSitesVec(i);
end


% Matrix for storing the sampled results.
X = zeros(size(T,1), totNumSites);

% current state defined by the copy number of each components is save in x.
if equillibrated==0 % Association experiment
    x0=zeros(1,totNumSites);
end


%Ti defines an ndex of where in T one should write a sample
Ti=1;


% simulation time is kept in t.
t = 0;

x=x0;
Ti=1;

%Intitiate binding sites positions
positions=zeros(totNumSites,2)

for i=1:totNumSites
    
    positions(i,1)=xDim*rand();
    positions(i,2)=yDim*rand();
    
end



% The simulation will be stopped after the system state has been saved at
% the last time point in T.
is_running=true;
while is_running
    % r is a vector containing the rate for each reaction
    r =zeros(1,2*totNumSites);
    k=1;
    for i=1:2:totNumSites
        if x(k)==1 %Dissocation possible
            r(i)=kdVecAll(k);
        elseif x(k)==0
            r(i+1)=kaVecAll(k);% assocation possible
        end
    end
    
    
    %sample the time until any evet will happen
    trxn = exprnd(1/sum(r));
    
    t = t + trxn;
    
    %Save state at regular intervals
    while t>= T(Ti)
        X(Ti,:) =x;
        Ti=Ti+1;
        if Ti == length(T) +1
              % Stop the simulation
            is_running = false;
   
    %        %Make sure to break the current while loop (otherwise T(Ti)
    %        % will point outside the vector.
            break;
        end
    end
    
    
    % sample which reaction that occured.
    sampled_rxn = find(rand() <= cumsum(r)./sum(r),1);
    
    % update the state according the sampled reaction the change is defined
    % by the matrix S.
    x = x + S(sampled_rxn,:);
    
    if t>tmax
        is_running=false;
    end
    
end




end

