function [kdFromSlope,kaFromSlope,intensityAllTime,intensityFirstTime,newOffCurve,bestParams] = getRatesIntensityMethodver4(traces,timeVec,startTimeAllTimes,timeToUseAllTimes,protconc,intBounds,minLevel,maxLevel,fitPointsSlopeoff,fitPointsSlopeon)
%This function estiamtes on and offrates based on fitting a slope to the
%sum of the fluorescence intensity, reconstructed from single molecule data. 
%Slope in the beginning of curve is used to estimate the on and offrate
%ver2: Correcting for the fact that there can be some amount of nonspecific
%binding
%ver3: Adding fit to ODemodel to correct for soem amount of nonspecific
%binding. If parameter kb is set to inf, kd is assumed to be 0 and
%kleaching is the fitted parameter.
%ver4: Combined fitting on on and off rates curve simultaunously

[~,startIndAll]=min(abs(timeVec-startTimeAllTimes));
[~,endInd]=min(abs(timeVec-timeToUseAllTimes));
numTraces=numel(traces);
numBoundsCorr=numel(intBounds);
numTimePoints=numel(traces{1});
intensityFirstTime=zeros(numBoundsCorr,numTimePoints);
intensityAllTime=zeros(numBoundsCorr,endInd);
numOffTraces=0;
doCorrection=1;
for i=1:numTraces
    
    intenseFirst=(traces{i}(1));
    trace=traces{i}';
    trace(trace<minLevel)=minLevel;
    trace(trace>maxLevel)=maxLevel;
    
    [~,ind]=min(abs(intBounds-intenseFirst));
    intensityFirstTime(ind,:)=intensityFirstTime(ind,:)+trace;
    for j=startIndAll:numTimePoints-endInd+1
        [~,ind]=min(abs(intBounds-trace(j)));
        intensityAllTime(ind,1:endInd)=intensityAllTime(ind,1:endInd)+trace(j:j+endInd-1);%nansum([autoCorrsAllTime(ind,:);autcorrs2d{i}(j,:)]);
        if ind==numel(intBounds)
            %break;
            numOffTraces=numOffTraces+1;
        end
    end
end
if maxLevel<inf
    intensityFirstTime=intensityFirstTime/(numTraces*maxLevel);
else
    intensityFirstTime=intensityFirstTime/numTraces;
    
end
%p=polyfit(timeVec(1:fitPointsSlopeon),intensityFirstTime(1,1:fitPointsSlopeon),1);
%slopeon=p(1);
%kaFromSlope=slopeon/protconc;



if doCorrection==1
    endInd=endInd-1;
    intensityAllTimeNew=zeros(numBoundsCorr,endInd);
    for i=1:numBoundsCorr
        correction=movmean(intensityAllTime(i,:),2);
        intensityAllTimeNew(i,:)=[intensityAllTime(i,1) correction(3:end)];
    end
    
    intensityAllTime=intensityAllTimeNew;
end

%p=polyfit(timeVec(1:fitPointsSlopeoff),intensityAllTime(end,1:fitPointsSlopeoff)/intensityAllTime(end,1),1);
% p=polyfit(timeVec(1:fitPointsSlopeoff),intensityAllTime(end,1:2:2*fitPointsSlopeoff-1)/intensityAllTime(end,1),1);
%slopeoff=p(1);
%kdFromSlope=-slopeoff/(numOffTraces*maxLevel);
%kdFromSlopeOld=-slopeoff;


timesOn=timeVec(1:fitPointsSlopeon);

intenseFuncOn = @(x) (x(2)*protconc/(x(2)*protconc+x(1)))*(1-exp(-((x(1)+x(2)*protconc)*timesOn))); %Also add a constant offset x(2) and factor x(3) to account for that the normalization with the point at time zero can be influenced to some small extent by the shut-noise

timesOff=timeVec(1:endInd);

intenseFuncOff = @(x) (x(2)*protconc/(x(2)*protconc+x(1))+(x(1)/(x(2)*protconc+x(1)))*exp(-((x(1)+x(2)*protconc)*timesOff)))*(x(3))+(1-x(3));

funDiff= @(params) sum((intensityAllTime(end,:)/(intensityAllTime(end,1))-intenseFuncOff(params)).^2)+sum((intensityFirstTime(1,1:fitPointsSlopeon)-intenseFuncOn(params)).^2);

lb=[1e-5 1e-5 0]; %[kd ka maximalFluorescensesignalfromDNA ]
ub=[1 1 1 ];

guessMinLog10=[log10(lb(1)) log10(lb(2)) -5];
guessMaxLog10=[log10(ub(1)) log10(ub(2)) 0];




options=optimoptions('fmincon','Display','off','TolX',1e-5,'TolFun',1e-5);
numGuesses=5;
paramsAll=zeros(numGuesses,numel(lb));
resnormAll=zeros(numGuesses,1);
for i= 1:numGuesses
    
    params0=10.^(guessMinLog10+rand(1,3).*(guessMaxLog10-guessMinLog10));
    
    
    
    [paramsCurr,resnorm] = fmincon(funDiff,params0,[],[],[],[],lb,ub,[],options);
    resnormAll(i)=resnorm;
    paramsAll(i,:)=paramsCurr;
    
    
end

[bestResNorm,bestind]=min(resnormAll);
bestParams=paramsAll(bestind,:);
newOffCurve=intensityAllTime(end,:)/(intensityAllTime(end,1))-(1-bestParams(3)); %Remove the background
newOffCurve=newOffCurve/newOffCurve(1);

kdFromSlope=bestParams(1);
kaFromSlope=bestParams(2);



% numGuess=10;
% 
% %[funcValues] = solveLinearOdes(timeVec,A,b,xzeros)
% if kbleaching==inf
%      lb=[1e-10 kaFromSlope 1e-5 0]; %[kd ka kbleaching maximalFluorescensesignalfromDNA ]
%     ub=[1e-10 kaFromSlope 1 1 ];
%     
% else
%     lb=[1e-5 kaRef kbleaching 0]; %[kd ka kbleaching maximalFluorescensesignalfromDNA ]
%     ub=[1 kaRef kbleaching 1 ];
% end
% 
% guessMinLog10=[log10(lb(1)) log10(lb(2)) log10(lb(3)) -5];
% guessMaxLog10=[log10(ub(1)) log10(ub(2)) log10(ub(3)) 0];
% % 
% timeStep=timeVec(2)-timeVec(1);
% timeVecOff=0:timeStep:(numel(intensityAllTime(end,:))-1)*timeStep;
% numNewTimepoints=numel(timeVecOff);
% 
% intenseFunc = @(x) sum(solveLinearOdes(timeVecOff,[-(x(1)+x(2)*protconc+x(3)) -x(2)*protconc;x(3) -x(1)],[x(2)*protconc;0],[x(4);0]).*[ones(1,numNewTimepoints);zeros(1,numNewTimepoints)],1)+(1-x(4));
% 
% 
% funDiff= @(params) sum((intensityAllTime(end,:)/(intensityAllTime(end,1))-intenseFunc(params)).^2);
% options=optimoptions('fmincon','Display','off','TolX',1e-3,'TolFun',1e-3);
% numGuesses=5;
% for i= 1:numGuesses
%     
%     params0=10.^(guessMinLog10+rand(1,4).*(guessMaxLog10-guessMinLog10));
%     
%     
%     
%     [paramsCurr,resnorm] = fmincon(funDiff,params0,[],[],[],[],lb,ub,[],options);
%     resnormAll(i)=resnorm;
%     paramsAll(i,:)=paramsCurr;
%     
%     
% end
% 
% [bestResNorm,bestind]=min(resnormAll);
% bestParams=paramsAll(bestind,:);
% 
% newOffCurve=intensityAllTime(end,:)/(intensityAllTime(end,1))-(1-bestParams(4)); %Remove the background
% newOffCurve=newOffCurve/newOffCurve(1);
% 
% 
% p=polyfit(timeVec(1:fitPointsSlopeoff),newOffCurve(end,1:fitPointsSlopeoff),1);
% 
% slopeoff=p(1);
% 
% kdFromSlope=-slopeoff;



end

